﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Verse;
using RimWorld;
using UnityEngine;


namespace AIRyrtwofAddon.Information
{
    public class InfoWindow : MainTabWindow
    {
        public static InfoWindow Instance { get; } = new InfoWindow();


        public bool opend = false;

        public ThingDef selected = null;
        public CompProperties_AbnosInfo info = null;

        private readonly float textSize = Text.LineHeight;

        private Vector2 storyScrollPos = Vector2.zero;

        public override Vector2 InitialSize => new Vector2(500,480);

        private bool isStory;
        private List<TabRecord> tabs;


        public InfoWindow()
        {
            draggable = true;
            //closeOnClickedOutside = true;
            onlyOneOfTypeAllowed = true;
            layer = WindowLayer.Dialog;
            doCloseX = true;
            tabs = new List<TabRecord>();
        }

        public override void PreOpen()
        {
            base.PreOpen();
            opend = true;
            storyScrollPos = Vector2.zero;
            info = selected.GetCompProperties<CompProperties_AbnosInfo>();
            
            isStory = false;

            tabs.Clear();
            tabs.Add(new TabRecord("管理情報", () => {
                isStory = false;
                storyScrollPos = Vector2.zero;
            }, () => !isStory));
            tabs.Add(new TabRecord("物語", () => {
                isStory = true;
                storyScrollPos = Vector2.zero;
            }, () => isStory));
        }

        public override void PostClose()
        {
            base.PostClose();
            selected = null;
            opend = false;
            info = null;

            isStory = false;
        }

        public override void DoWindowContents(Rect inRect)
        {
            if (null == selected) return;
            if (info == null)
                info = selected.GetCompProperties<CompProperties_AbnosInfo>();

            GameFont font = Text.Font;

            Rect mainCnt = inRect;
            mainCnt.width = 260f;
            MainContent(mainCnt);
            
            Rect tabCnt = inRect;
            tabCnt.width = InitialSize.x - mainCnt.width - 42f;
            tabCnt.x = mainCnt.xMax +5f;
            WindowTool.BoxTab(tabCnt, tabs);

            tabCnt.yMin += 31f;
            if (isStory) StoryContent(tabCnt);
            else InfoContent(tabCnt);

            Text.Font = font;
        }

        private void MainContent(Rect rect)
        {
            float length = Math.Max(
                Text.CalcSize(info.classID).x + 10f +
                Text.CalcSize(info.danger.ToString()).x + 8f,
                260f);

            Text.Font = GameFont.Small;
            Rect label = rect;
            label.width = Math.Max(Text.CalcSize(selected.label).x + 2f, length);
            label.height = Text.CalcSize(selected.label).y - 2f;
            WindowTool.TextCentering(label, selected.label);
            Widgets.DrawLineHorizontal(label.xMin, label.yMax, label.width);
            

            Rect classID = rect.TextArea();
            classID.y += label.height + 3f;
            classID.width = Text.CalcSize(info.classID).x;
            Widgets.Label(classID, info.classID);

            classID.width = Text.CalcSize(info.danger.ToString()).x +8f;
            classID.xMin = label.xMax - Text.CalcSize(info.danger.ToString()).x - 10f;
            WindowTool.FeatureLabel(classID,
                new StringColor(info.danger.ToString(),
                    InfoContentCache.dangerColor[(int)info.danger])
            );
            
            Rect imageRect = rect.Square(150f);
            imageRect.y = classID.yMax + 5f;
            Widgets.DrawTextureFitted(imageRect, selected.uiIcon, selected.uiIconScale);
            Widgets.DrawBox(imageRect, 2);

            Rect egoRect = imageRect.Square(100f);
            egoRect.y = imageRect.yMax - egoRect.height;
            egoRect.x += imageRect.width + 5f;
            Widgets.DrawBox(egoRect);
            if(info.ego != null)
            {
                Widgets.DrawTextureFitted(egoRect, info.ego.uiIcon, info.ego.uiIconScale);
                WindowTool.TooltipDescription(egoRect,info.ego.label);
            }
            else  WindowTool.TextCentering(egoRect, "No Data");

                Rect egoNameRect = egoRect.TextArea();
            egoNameRect.y -= textSize;
            WindowTool.TextCentering(egoNameRect, "EGO");

            Rect elCapRect = label.TextArea();
            elCapRect.xMin = elCapRect.xMax - Text.CalcSize(info.ELCap.ToString()).x;
            elCapRect.y = classID.yMax + 3f;
            WindowTool.ColorAction(Color.yellow, () => {
                Widgets.Label(elCapRect, info.ELCap);
            });
            
            Rect fetRect = label.TextArea();
            fetRect.y = imageRect.yMax + 5f;
            for (int i = 0; i < Math.Min(info.AllFeatures.Count, 6); i++)
            {
                WindowTool.FeatureLabel(fetRect, info.AllFeatures[i]);
                //fetRect.y += textSize + 7f;
                fetRect.x += Text.CalcSize(info.AllFeatures[i].tag).x + 15f;
            }

            Rect exDescRect = rect;
            exDescRect.yMin = fetRect.yMax + 8f;
            Widgets.DrawMenuSection(exDescRect);

            Rect descRect = exDescRect.TextArea();
            descRect.x += 5f;
            foreach (string str in info.exDesc)
            {
                descRect.height = Text.CalcHeight(str,descRect.width - 15f);
                Widgets.Label(descRect, "・" + str);
                descRect.y += descRect.height;
            }

        }

        private void InfoContent(Rect rect)
        {
            Rect viewRect = new Rect(rect.x, rect.y, rect.width - textSize, 0);
            viewRect.height = info.ExInfoSize(viewRect.width-20f);

            Rect cntRect = viewRect;
            Widgets.BeginScrollView(rect, ref storyScrollPos, viewRect);
            foreach(ImageDesc id in info.exInfo)
            {
                cntRect.height = id.Height(viewRect.width - 20f);
                InfoMenu(cntRect, id);
                cntRect.y += cntRect.height;
            }
            Widgets.EndScrollView();
        }

        private void StoryContent(Rect rect)
        {
            Rect viewRect = new Rect(rect.x,rect.y,rect.width - textSize, 0);

            foreach (string str in info.story)
            {
                viewRect.height += Text.CalcHeight(str, viewRect.width - 20f) + 16f;
            }

            Rect cntRect = viewRect;

            Widgets.BeginScrollView(rect, ref storyScrollPos, viewRect);
            for(int i = 0; i < info.story.Count; i++)
            {
                cntRect.height = Text.CalcHeight(info.story[i], viewRect.width - 20f) + 16f;
                StoryMenu(cntRect, info.story[i]);
                cntRect.y += cntRect.height;
                if (i < info.story.Count) 
                    Widgets.DrawLineHorizontal(cntRect.x + 6f,cntRect.y, cntRect.width -8f);
            }
            Widgets.EndScrollView();
        }

        private void StoryMenu(Rect rect, string str)
        {
            Rect label = rect.ContractedBy(4f);
            label.xMin += 2f;
            label.xMax -= 2f;
            Widgets.Label(label, str);
        }

        private void InfoMenu(Rect rect, ImageDesc id)
        {
            //Widgets.DrawBox(rect);

            Rect label = rect.ContractedBy(8f);
            label.xMin += 2f;  
            label.xMax -= 2f;
            Widgets.Label(label, id.desc);

            if (id.imagePath !=null)
            {
                if(!id.desc.NullOrEmpty())
                    label.yMin += Text.CalcHeight(id.desc, label.width) + 10f;
                Widgets.DrawTextureFitted(label,id.Get(),1f);
                
            }

        }


        public void Open(ThingDef thing)
        {
            selected = thing;
            storyScrollPos = Vector2.zero;

            if (!opend)  Find.WindowStack.Add(this);
        }

    }
}
